#ifndef __HPCXX_COMMCOORD_H_
#define __HPCXX_COMMCOORD_H_

#include "HPCxx_Comm.h"
#include "HPCxx_Mutex.h"
#include <vector.h>
#include <deque.h>
#include <list.h>

class HPCxx_CommCoord {

 public:

  //#---- HPCxx_CommCoord -----------------------------------------------------
  // Constructs a communications coordinator which builds communication objects
  // and coordinates and facilitates communications.
  //#--------------------------------------------------------------------------
  HPCxx_CommCoord(void);

  //#---- ~HPCxx_CommCoord ----------------------------------------------------
  // Destroys this communications coordinator.
  //#--------------------------------------------------------------------------
  ~HPCxx_CommCoord(void) = 0;

  //#---- communicator --------------------------------------------------------
  // Returns a communications object.
  // <ul>
  // <li><b>dest</b> -- the destination of the message
  // <li><b>msgType</b> -- the message type to send
  // <li><b>unique</b> -- use a unique message id if true
  // </ul>
  //#--------------------------------------------------------------------------
  virtual HPCxx_Comm* communicator(const HPCxx_ContextID& dest, int msgType,
				   bool unique = false) = 0;

  //#---- handleMessage -------------------------------------------------------
  // Handles an incoming message.
  // <ul>
  // <li><b>comm</b> -- the communication object containing the message
  //#--------------------------------------------------------------------------
  virtual void handleMessage(HPCxx_Comm& comm);

  //#---- release -------------------------------------------------------------
  // Releases what had been an "in use" communications object.
  // <ul>
  // <li><b>comm</b> -- the communication object to release
  // </ul>
  //#--------------------------------------------------------------------------
  virtual void release(HPCxx_Comm* comm) = 0;

  //#---- commMutex -----------------------------------------------------------
  // Returns the communication mutex to be used to protect communication
  // critical sections.
  //#--------------------------------------------------------------------------
  static HPCxx_Mutex* commMutex(void);

  //#---- setCommMutex --------------------------------------------------------
  // Sets the communication mutex to be used to protect communication critical
  // sections.
  //#--------------------------------------------------------------------------
  static void setCommMutex(HPCxx_Mutex* mutex);

  //#---- poll ----------------------------------------------------------------
  // Polls communication queues for events.
  //#--------------------------------------------------------------------------
  virtual void poll(void) = 0;

 private:

  static HPCxx_Mutex* commMutex_;	// mutex to protect communication

};

#endif // __HPCXX_COMMCOORD_H_
