#include <sys/utsname.h>
#include <stdlib.h>
#include <strings.h>
#include "templ.h"
#include "HPCxx_String.h"
#include "MyArray.h"
#include <hpcxx_rts.h>
#include <unistd.h>

// To wait until killed by the client
HPCxx_Sync<int> *mywait;
int waitOnMe = 1;
// Ignore the command line arguments that are being passed on by Globus,
// just print the host on which this server is running. ( Globus sends a
// set of arguments when the new context is created ).

void parseArguments(int argc, char **argv) {
  const int maxLengthOfName = 100;
  char localHostName [maxLengthOfName];

  int error = gethostname(localHostName, maxLengthOfName);
  if (error){
    cout << "Error ... gethostname() failed ... exiting \n";
    exit(EXIT_FAILURE);
  }
  cout << "Gramtest-Server is running on: " << localHostName << endl;
}

// ids used in server for its externally called functions
#define PASS_ARRAY_ID        179
#define PASS_DATA_ID         180
#define SAY_HELLO_ID         181
#define KILL_ME_ID           182
int counter = 0;
// which correspond to these functions
float passData(float x, int y, int z){
  cout << counter++ << endl << flush;
  float w = x + y + z;
  return w;
}

int  passArray(MyArray a){
   int l = a.getLength();
   //cout << "got it!" << l <<  endl;
   /*
   cout << "{";
   for(int j = 0; j < l; j++){
     cout << a[j];
     if(j < l-1) cout << ", ";
     }
   cout << "}\n";
   */
   //cout.flush();
   return l;
}

HPCxx_String sayHello(HPCxx_String source){ 
  while (waitOnMe) ;
  cout << "Server::sayHello request received from: " << source << endl;
  cout.flush();
  return HPCxx_String("Hello from Server !!\n");
}


int killMe(){
  cout << "Server ::killMe called\n";
  cout.flush();

  // Writes to the Sync variable so that the main thread waiting for "read"
  // from the Sync variable can continue thus finishing the server context.
  int x = 1;
  *mywait = x;
  return x;  // return some dummy value
}

// Start running the server
void runServer(int argc, char **argv) {
  // Register all the functions that the server context exports.
  hpcxx_register(passArray, PASS_ARRAY_ID);
  hpcxx_register(passData, PASS_DATA_ID);
  hpcxx_register(sayHello, SAY_HELLO_ID);
  hpcxx_register(killMe, KILL_ME_ID);
  
  // Wait until killed by the client.
  mywait = new HPCxx_Sync<int>;

  waitOnMe = 0;  // I am done with all my initialization

  int killMe = *mywait;
  
  cout << "Component has been killed! \n ";
  cout << "Server exiting ... \n";
  cout.flush();
  
  // Wait for a few seconds for the killMe request from the client to
  // return back.
  sleep(5);
}

// The main function.
int main(int argc, char **argv) {
  // Required for any context created using hpcxx_createContext. The newly
  // created context is essentially an "agent" for the "creator" context. 
  hpcxx_initAgent(argc, argv);

  cout << "Server: in main(): hpcxx_initAgent done \n";
  cout.flush(); 

  parseArguments(argc, argv);
  runServer(argc, argv);

  // Call this before exiting. Used for cleanup.  
  hpcxx_exit();
}







